@@ -84,7 +84,7 @@ class VComment(mutagen.Metadata, list):
             vendor_length = cdata.uint_le(fileobj.read(4))
             self.vendor = fileobj.read(vendor_length).decode('utf-8', errors)
             count = cdata.uint_le(fileobj.read(4))
-            for i in range(count):
+            for i in xrange(count):
                 length = cdata.uint_le(fileobj.read(4))
                 try: string = fileobj.read(length).decode('utf-8', errors)
                 except (OverflowError, MemoryError):
@@ -220,4 +220,4 @@ class VCommentDict(VComment, DictMixin):
 
     def as_dict(self):
         """Return a copy of the comment data in a real dict."""
-        return dict((key, self[key]) for key in self.keys())
+        return dict([(key, self[key]) for key in self.keys()])
