@@ -115,6 +115,9 @@ class ASFBaseAttribute(object):
         else:
             self.value = value
 
+    def data_size(self):
+        raise NotImplementedError
+
     def __repr__(self):
         name = "%s(%r" % (type(self).__name__, self.value)
         if self.language:
@@ -158,12 +161,17 @@ class ASFUnicodeAttribute(ASFBaseAttribute):
     def _render(self):
         return self.value.encode("utf-16-le") + "\x00\x00"
 
+    def data_size(self):
+        return len(self._render())
+
     def __str__(self):
         return self.value
 
     def __cmp__(self, other):
         return cmp(unicode(self), other)
 
+    __hash__ = ASFBaseAttribute.__hash__
+
 
 class ASFByteArrayAttribute(ASFBaseAttribute):
     """Byte array attribute."""
@@ -175,12 +183,17 @@ class ASFByteArrayAttribute(ASFBaseAttribute):
     def _render(self):
         return self.value
 
+    def data_size(self):
+        return len(self.value)
+
     def __str__(self):
         return "[binary data (%s bytes)]" % len(self.value)
 
     def __cmp__(self, other):
         return cmp(str(self), other)
 
+    __hash__ = ASFBaseAttribute.__hash__
+
 
 class ASFBoolAttribute(ASFBaseAttribute):
     """Bool attribute."""
@@ -198,6 +211,9 @@ class ASFBoolAttribute(ASFBaseAttribute):
         else:
             return struct.pack("<H", int(self.value))
 
+    def data_size(self):
+        return 4
+
     def __bool__(self):
         return self.value
 
@@ -207,6 +223,8 @@ class ASFBoolAttribute(ASFBaseAttribute):
     def __cmp__(self, other):
         return cmp(bool(self), other)
 
+    __hash__ = ASFBaseAttribute.__hash__
+
 
 class ASFDWordAttribute(ASFBaseAttribute):
     """DWORD attribute."""
@@ -218,6 +236,9 @@ class ASFDWordAttribute(ASFBaseAttribute):
     def _render(self):
         return struct.pack("<L", self.value)
 
+    def data_size(self):
+        return 4
+
     def __int__(self):
         return self.value
 
@@ -227,6 +248,8 @@ class ASFDWordAttribute(ASFBaseAttribute):
     def __cmp__(self, other):
         return cmp(int(self), other)
 
+    __hash__ = ASFBaseAttribute.__hash__
+
 
 class ASFQWordAttribute(ASFBaseAttribute):
     """QWORD attribute."""
@@ -238,6 +261,9 @@ class ASFQWordAttribute(ASFBaseAttribute):
     def _render(self):
         return struct.pack("<Q", self.value)
 
+    def data_size(self):
+        return 8
+
     def __int__(self):
         return self.value
 
@@ -247,6 +273,8 @@ class ASFQWordAttribute(ASFBaseAttribute):
     def __cmp__(self, other):
         return cmp(int(self), other)
 
+    __hash__ = ASFBaseAttribute.__hash__
+
 
 class ASFWordAttribute(ASFBaseAttribute):
     """WORD attribute."""
@@ -258,6 +286,9 @@ class ASFWordAttribute(ASFBaseAttribute):
     def _render(self):
         return struct.pack("<H", self.value)
 
+    def data_size(self):
+        return 2
+
     def __int__(self):
         return self.value
 
@@ -267,6 +298,8 @@ class ASFWordAttribute(ASFBaseAttribute):
     def __cmp__(self, other):
         return cmp(int(self), other)
 
+    __hash__ = ASFBaseAttribute.__hash__
+
 
 class ASFGUIDAttribute(ASFBaseAttribute):
     """GUID attribute."""
@@ -278,12 +311,17 @@ class ASFGUIDAttribute(ASFBaseAttribute):
     def _render(self):
         return self.value
 
+    def data_size(self):
+        return len(self.value)
+
     def __str__(self):
         return self.value
 
     def __cmp__(self, other):
         return cmp(str(self), other)
 
+    __hash__ = ASFBaseAttribute.__hash__
+
 
 UNICODE = ASFUnicodeAttribute.TYPE
 BYTEARRAY = ASFByteArrayAttribute.TYPE
@@ -540,7 +578,7 @@ class ASF(FileType):
 
     def load(self, filename):
         self.filename = filename
-        fileobj = file(filename, "rb")
+        fileobj = open(filename, "rb")
         try:
             self.size = 0
             self.size1 = 0
@@ -562,11 +600,13 @@ class ASF(FileType):
         for name, value in self.tags:
             if name in _standard_attribute_names:
                 continue
+            library_only = (value.data_size() > 0xFFFF or value.TYPE == GUID)
             if (value.language is None and value.stream is None and
-                name not in self.to_extended_content_description):
+                name not in self.to_extended_content_description and
+                not library_only):
                 self.to_extended_content_description[name] = value
             elif (value.language is None and value.stream is not None and
-                  name not in self.to_metadata):
+                  name not in self.to_metadata and not library_only):
                 self.to_metadata[name] = value
             else:
                 self.to_metadata_library.append((name, value))
@@ -599,7 +639,7 @@ class ASF(FileType):
                 struct.pack("<QL", len(data) + 30, len(self.objects)) +
                 "\x01\x02" + data)
 
-        fileobj = file(self.filename, "rb+")
+        fileobj = open(self.filename, "rb+")
         try:
             size = len(data)
             if size > self.size:
@@ -611,6 +651,9 @@ class ASF(FileType):
         finally:
             fileobj.close()
 
+        self.size = size
+        self.num_objects = len(self.objects)
+
     def __read_file(self, fileobj):
         header = fileobj.read(30)
         if len(header) != 30 or header[:16] != HeaderObject.GUID:
