@@ -27,6 +27,9 @@ from _vorbis import VCommentDict
 from mutagen import FileType
 from mutagen._util import insert_bytes
 from mutagen.id3 import BitPaddedInt
+import sys
+if sys.version_info >= (2, 6):
+    from functools import reduce
 
 class error(IOError): pass
 class FLACNoHeaderError(error): pass
@@ -37,6 +40,26 @@ def to_int_be(string):
     byte order."""
     return reduce(lambda a, b: (a << 8) + ord(b), string, 0L)
 
+class StrictFileObject(object):
+    """Wraps a file-like object and raises an exception if the requested
+    amount of data to read isn't returned."""
+
+    def __init__(self, fileobj):
+        self._fileobj = fileobj
+        for m in ["close", "tell", "seek", "write", "name"]:
+            if hasattr(fileobj, m):
+                setattr(self, m, getattr(fileobj, m))
+
+    def read(self, size=-1):
+        data = self._fileobj.read(size)
+        if size >= 0 and len(data) != size:
+            raise error("file said %d bytes, read %d bytes" %(
+                        size, len(data)))
+        return data
+
+    def tryread(self, *args):
+        return self._fileobj.read(*args)
+
 class MetadataBlock(object):
     """A generic block of FLAC metadata.
 
@@ -47,14 +70,19 @@ class MetadataBlock(object):
     data -- raw binary data for this block
     """
 
+    _distrust_size = False
+
     def __init__(self, data):
         """Parse the given data string or file-like as a metadata block.
         The metadata header should not be included."""
         if data is not None:
-            if isinstance(data, str): data = StringIO(data)
-            elif not hasattr(data, 'read'):
-                raise TypeError(
-                    "StreamInfo requires string data or a file-like")
+            if not isinstance(data, StrictFileObject):
+                if isinstance(data, str):
+                    data = StringIO(data)
+                elif not hasattr(data, 'read'):
+                    raise TypeError(
+                        "StreamInfo requires string data or a file-like")
+                data = StrictFileObject(data)
             self.load(data)
 
     def load(self, data): self.data = data.read()
@@ -118,6 +146,7 @@ class StreamInfo(MetadataBlock):
                      self.bits_per_sample == other.bits_per_sample and
                      self.total_samples == other.total_samples)
         except: return False
+    __hash__ = MetadataBlock.__hash__
 
     def load(self, data):
         self.min_blocksize = int(to_int_be(data.read(2)))
@@ -133,6 +162,8 @@ class StreamInfo(MetadataBlock):
 
         sample_tail = sample_channels_bps >> 4
         self.sample_rate = int((sample_first << 4) + sample_tail)
+        if not self.sample_rate:
+            raise error("A sample rate value of 0 is invalid")
         self.channels = int(((sample_channels_bps >> 1) & 7) + 1)
         bps_tail = bps_total >> 36
         bps_head = (sample_channels_bps & 1) << 4
@@ -214,14 +245,15 @@ class SeekTable(MetadataBlock):
     def __eq__(self, other):
         try: return (self.seekpoints == other.seekpoints)
         except (AttributeError, TypeError): return False
+    __hash__ = MetadataBlock.__hash__
 
     def load(self, data):
         self.seekpoints = []
-        sp = data.read(self.__SEEKPOINT_SIZE)
+        sp = data.tryread(self.__SEEKPOINT_SIZE)
         while len(sp) == self.__SEEKPOINT_SIZE:
             self.seekpoints.append(SeekPoint(
                 *struct.unpack(self.__SEEKPOINT_FORMAT, sp)))
-            sp = data.read(self.__SEEKPOINT_SIZE)
+            sp = data.tryread(self.__SEEKPOINT_SIZE)
 
     def write(self):
         f = StringIO()
@@ -243,6 +275,7 @@ class VCFLACDict(VCommentDict):
     """
 
     code = 4
+    _distrust_size = True
 
     def load(self, data, errors='replace', framing=False):
         super(VCFLACDict, self).load(data, errors=errors, framing=framing)
@@ -304,6 +337,7 @@ class CueSheetTrack(object):
                      self.pre_emphasis == other.pre_emphasis and
                      self.indexes == other.indexes)
         except (AttributeError, TypeError): return False
+    __hash__ = object.__hash__
 
     def __repr__(self):
         return ("<%s number=%r, offset=%d, isrc=%r, type=%r, "
@@ -350,6 +384,7 @@ class CueSheet(MetadataBlock):
                      self.compact_disc == other.compact_disc and
                      self.tracks == other.tracks)
         except (AttributeError, TypeError): return False
+    __hash__ = MetadataBlock.__hash__
 
     def load(self, data):
         header = data.read(self.__CUESHEET_SIZE)
@@ -422,6 +457,7 @@ class Picture(MetadataBlock):
     """
 
     code = 6
+    _distrust_size = True
 
     def __init__(self, data=None):
         self.type = 0
@@ -444,6 +480,7 @@ class Picture(MetadataBlock):
                      self.colors == other.colors and
                      self.data == other.data)
         except (AttributeError, TypeError): return False
+    __hash__ = MetadataBlock.__hash__
 
     def load(self, data):
         self.type, length = struct.unpack('>2I', data.read(8))
@@ -496,6 +533,7 @@ class Padding(MetadataBlock):
             raise error("cannot write %d bytes" % self.length)
     def __eq__(self, other):
         return isinstance(other, Padding) and self.length == other.length
+    __hash__ = MetadataBlock.__hash__
     def __repr__(self):
         return "<%s (%d bytes)>" % (type(self).__name__, self.length)
 
@@ -521,31 +559,50 @@ class FLAC(FileType):
                 filename.lower().endswith(".flac") * 3)
     score = staticmethod(score)
 
-    def __read_metadata_block(self, file):
-        byte = ord(file.read(1))
-        size = to_int_be(file.read(3))
+    def __read_metadata_block(self, fileobj):
+        byte = ord(fileobj.read(1))
+        size = to_int_be(fileobj.read(3))
+        code = byte & 0x7F
+        last_block = bool(byte & 0x80)
+
         try:
-            data = file.read(size)
-            if len(data) != size:
-                raise error(
-                    "file said %d bytes, read %d bytes" % (size, len(data)))
-            block = self.METADATA_BLOCKS[byte & 0x7F](data)
-        except (IndexError, TypeError):
-            block = MetadataBlock(data)
-            block.code = byte & 0x7F
-            self.metadata_blocks.append(block)
+            block_type = self.METADATA_BLOCKS[code] or MetadataBlock
+        except IndexError:
+            block_type = MetadataBlock
+
+        if block_type._distrust_size:
+            # Some jackass is writing broken Metadata block length
+            # for Vorbis comment blocks, and the FLAC reference
+            # implementaton can parse them (mostly by accident),
+            # so we have to too.  Instead of parsing the size
+            # given, parse an actual Vorbis comment, leaving
+            # fileobj in the right position.
+            # http://code.google.com/p/mutagen/issues/detail?id=52
+            # ..same for the Picture block:
+            # http://code.google.com/p/mutagen/issues/detail?id=106
+            block = block_type(fileobj)
         else:
-            self.metadata_blocks.append(block)
-            if block.code == VCFLACDict.code:
-                if self.tags is None: self.tags = block
-                else: raise FLACVorbisError("> 1 Vorbis comment block found")
-            elif block.code == CueSheet.code:
-                if self.cuesheet is None: self.cuesheet = block
-                else: raise error("> 1 CueSheet block found")
-            elif block.code == SeekTable.code:
-                if self.seektable is None: self.seektable = block
-                else: raise error("> 1 SeekTable block found")
-        return (byte >> 7) ^ 1
+            data = fileobj.read(size)
+            block = block_type(data)
+        block.code = code
+
+        if block.code == VCFLACDict.code:
+            if self.tags is None:
+                self.tags = block
+            else:
+                raise FLACVorbisError("> 1 Vorbis comment block found")
+        elif block.code == CueSheet.code:
+            if self.cuesheet is None:
+                self.cuesheet = block
+            else:
+                raise error("> 1 CueSheet block found")
+        elif block.code == SeekTable.code:
+            if self.seektable is None:
+                self.seektable = block
+            else:
+                raise error("> 1 SeekTable block found")
+        self.metadata_blocks.append(block)
+        return not last_block
 
     def add_tags(self):
         """Add a Vorbis comment block to the file."""
@@ -578,13 +635,11 @@ class FLAC(FileType):
         self.cuesheet = None
         self.seektable = None
         self.filename = filename
-        fileobj = file(filename, "rb")
+        fileobj = StrictFileObject(open(filename, "rb"))
         try:
             self.__check_header(fileobj)
             while self.__read_metadata_block(fileobj):
                 pass
-            if fileobj.read(2) != "\xff\xf8":
-                raise FLACNoHeaderError("End of metadata did not start audio")
         finally:
             fileobj.close()
 
@@ -617,58 +672,71 @@ class FLAC(FileType):
         if filename is None: filename = self.filename
         f = open(filename, 'rb+')
 
-        # Ensure we've got padding at the end, and only at the end.
-        # If adding makes it too large, we'll scale it down later.
-        self.metadata_blocks.append(Padding('\x00' * 1020))
-        MetadataBlock.group_padding(self.metadata_blocks)
+        try:
+            # Ensure we've got padding at the end, and only at the end.
+            # If adding makes it too large, we'll scale it down later.
+            self.metadata_blocks.append(Padding('\x00' * 1020))
+            MetadataBlock.group_padding(self.metadata_blocks)
 
-        header = self.__check_header(f)
-        available = self.__find_audio_offset(f) - header # "fLaC" and maybe ID3
-        data = MetadataBlock.writeblocks(self.metadata_blocks)
+            header = self.__check_header(f)
+            available = self.__find_audio_offset(f) - header # "fLaC" and maybe ID3
+            data = MetadataBlock.writeblocks(self.metadata_blocks)
 
-        # Delete ID3v2
-        if deleteid3 and header > 4:
-            available += header - 4
-            header = 4
+            # Delete ID3v2
+            if deleteid3 and header > 4:
+                available += header - 4
+                header = 4
 
-        if len(data) > available:
-            # If we have too much data, see if we can reduce padding.
-            padding = self.metadata_blocks[-1]
-            newlength = padding.length - (len(data) - available)
-            if newlength > 0:
-                padding.length = newlength
+            if len(data) > available:
+                # If we have too much data, see if we can reduce padding.
+                padding = self.metadata_blocks[-1]
+                newlength = padding.length - (len(data) - available)
+                if newlength > 0:
+                    padding.length = newlength
+                    data = MetadataBlock.writeblocks(self.metadata_blocks)
+                    assert len(data) == available
+
+            elif len(data) < available:
+                # If we have too little data, increase padding.
+                self.metadata_blocks[-1].length += (available - len(data))
                 data = MetadataBlock.writeblocks(self.metadata_blocks)
                 assert len(data) == available
 
-        elif len(data) < available:
-            # If we have too little data, increase padding.
-            self.metadata_blocks[-1].length += (available - len(data))
-            data = MetadataBlock.writeblocks(self.metadata_blocks)
-            assert len(data) == available
+            if len(data) != available:
+                # We couldn't reduce the padding enough.
+                diff = (len(data) - available)
+                insert_bytes(f, diff, header)
 
-        if len(data) != available:
-            # We couldn't reduce the padding enough.
-            diff = (len(data) - available)
-            insert_bytes(f, diff, header)
+            f.seek(header - 4)
+            f.write("fLaC" + data)
 
-        f.seek(header - 4)
-        f.write("fLaC" + data)
-
-        # Delete ID3v1
-        if deleteid3:
-            try: f.seek(-128, 2)
-            except IOError: pass
-            else:
-                if f.read(3) == "TAG":
-                    f.seek(-128, 2)
-                    f.truncate()
+            # Delete ID3v1
+            if deleteid3:
+                try: f.seek(-128, 2)
+                except IOError: pass
+                else:
+                    if f.read(3) == "TAG":
+                        f.seek(-128, 2)
+                        f.truncate()
+        finally:
+            f.close()
 
     def __find_audio_offset(self, fileobj):
         byte = 0x00
-        while not (byte >> 7) & 1:
+        while not (byte & 0x80):
             byte = ord(fileobj.read(1))
             size = to_int_be(fileobj.read(3))
-            fileobj.read(size)
+            try:
+                block_type = self.METADATA_BLOCKS[byte & 0x7F]
+            except IndexError:
+                block_type = None
+
+            if block_type and block_type._distrust_size:
+                # See comments in read_metadata_block; the size can't
+                # be trusted for Vorbis comment blocks and Picture block
+                block_type(fileobj)
+            else:
+                fileobj.read(size)
         return fileobj.tell()
 
     def __check_header(self, fileobj):
