@@ -12,6 +12,8 @@ import struct
 
 from mutagen.id3 import ID3FileType, BitPaddedInt, delete
 
+__all__ = ["MP3", "Open", "delete", "MP3"]
+
 class error(RuntimeError): pass
 class HeaderNotFoundError(error, IOError): pass
 class InvalidMPEGHeader(error, IOError): pass
@@ -85,7 +87,7 @@ class MPEGInfo(object):
             except struct.error: id3, insize = '', 0
             insize = BitPaddedInt(insize)
             if id3 == 'ID3' and insize > 0:
-                offset = insize
+                offset = insize + 10
             else: offset = 0
 
         # Try to find two valid headers (meaning, very likely MPEG data)
@@ -151,6 +153,9 @@ class MPEGInfo(object):
         if self.layer == 1:
             frame_length = (12 * self.bitrate / self.sample_rate + padding) * 4
             frame_size = 384
+        elif self.version >= 2 and self.layer == 3:
+            frame_length = 72 * self.bitrate / self.sample_rate + padding
+            frame_size = 576
         else:
             frame_length = 144 * self.bitrate / self.sample_rate + padding
             frame_size = 1152
@@ -159,13 +164,15 @@ class MPEGInfo(object):
             possible = frame_1 + frame_length
             if possible > len(data) + 4:
                 raise HeaderNotFoundError("can't sync to second MPEG frame")
-            frame_data = struct.unpack(">H", data[possible:possible + 2])[0]
+            try:
+                frame_data = struct.unpack(
+                    ">H", data[possible:possible + 2])[0]
+            except struct.error:
+                raise HeaderNotFoundError("can't sync to second MPEG frame")
             if frame_data & 0xFFE0 != 0xFFE0:
                 raise HeaderNotFoundError("can't sync to second MPEG frame")
 
-        frame_count = real_size / float(frame_length)
-        samples = frame_size * frame_count
-        self.length = samples / self.sample_rate
+        self.length = 8 * real_size / float(self.bitrate)
 
         # Try to find/parse the Xing header, which trumps the above length
         # and bitrate calculation.
@@ -184,8 +191,9 @@ class MPEGInfo(object):
                 self.sketchy = False
                 vbri_version = struct.unpack('>H', data[vbri + 4:vbri + 6])[0]
                 if vbri_version == 1:
-                    frame_count = struct.unpack('>I', data[vbri + 14:vbri + 18])[0]
-                    samples = frame_size * frame_count
+                    frame_count = struct.unpack(
+                        '>I', data[vbri + 14:vbri + 18])[0]
+                    samples = float(frame_size * frame_count)
                     self.length = (samples / self.sample_rate) or self.length
         else:
             # If a Xing header was found, this is definitely MPEG audio.
@@ -193,7 +201,7 @@ class MPEGInfo(object):
             flags = struct.unpack('>I', data[xing + 4:xing + 8])[0]
             if flags & 0x1:
                 frame_count = struct.unpack('>I', data[xing + 8:xing + 12])[0]
-                samples = frame_size * frame_count
+                samples = float(frame_size * frame_count)
                 self.length = (samples / self.sample_rate) or self.length
             if flags & 0x2:
                 bytes = struct.unpack('>I', data[xing + 12:xing + 16])[0]
@@ -221,3 +229,8 @@ class MP3(ID3FileType):
     score = staticmethod(score)
 
 Open = MP3
+
+class EasyMP3(MP3):
+    """Like MP3, but uses EasyID3 for tags."""
+    from mutagen.easyid3 import EasyID3 as ID3
+
