@@ -1,12 +1,12 @@
 # Ogg Speex support.
 #
-# Copyright 2006 Joe Wreschnig <piman@sacredchao.net>
+# Copyright 2006 Joe Wreschnig
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation.
 #
-# $Id: oggspeex.py 4275 2008-06-01 06:32:37Z piman $
+# $Id: oggspeex.py 3976 2007-01-13 22:00:14Z piman $
 
 """Read and write Ogg Speex comments.
 
@@ -51,11 +51,13 @@ class OggSpeexInfo(object):
                 "page has ID header, but doesn't start a stream")
         self.sample_rate = cdata.uint_le(page.packets[0][36:40])
         self.channels = cdata.uint_le(page.packets[0][48:52])
-        self.bitrate = cdata.int_le(page.packets[0][52:56])
-        if self.bitrate == -1:
-            self.bitrate = 0
+        self.bitrate = max(0, cdata.int_le(page.packets[0][52:56]))
         self.serial = page.serial
 
+    def _post_tags(self, fileobj):
+        page = OggPage.find_last(fileobj, self.serial)
+        self.length = page.position / float(self.sample_rate)
+
     def pprint(self):
         return "Ogg Speex, %.2f seconds" % self.length
 
