@@ -1,5 +1,7 @@
 import os
 from StringIO import StringIO
+from tempfile import mkstemp
+import shutil
 
 from tests import TestCase, add
 from mutagen import File, Metadata, FileType
@@ -7,6 +9,7 @@ from mutagen.oggvorbis import OggVorbis
 from mutagen.oggflac import OggFLAC
 from mutagen.oggspeex import OggSpeex
 from mutagen.oggtheora import OggTheora
+from mutagen.oggopus import OggOpus
 from mutagen.mp3 import MP3, EasyMP3
 from mutagen.apev2 import APEv2File
 from mutagen.flac import FLAC
@@ -73,7 +76,7 @@ class TFile(TestCase):
 
     def test_empty(self):
         filename = os.path.join("tests", "data", "empty")
-        file(filename, "wb").close()
+        open(filename, "wb").close()
         try: self.failUnless(File(filename) is None)
         finally: os.unlink(filename)
 
@@ -101,6 +104,10 @@ class TFile(TestCase):
         self.failUnless(isinstance(File(
             os.path.join("tests", "data", "sample.oggtheora")), OggTheora))
 
+    def test_oggopus(self):
+        self.failUnless(isinstance(File(
+            os.path.join("tests", "data", "example.opus")), OggOpus))
+
     def test_mp3(self):
         self.failUnless(isinstance(
             File(os.path.join("tests", "data", "bad-xing.mp3")), MP3))
@@ -125,6 +132,8 @@ class TFile(TestCase):
             File(os.path.join("tests", "data", "sv4_header.mpc")), Musepack))
         self.failUnless(isinstance(
             File(os.path.join("tests", "data", "sv5_header.mpc")), Musepack))
+        self.failUnless(isinstance(
+            File(os.path.join("tests", "data", "sv8_header.mpc")), Musepack))
 
     def test_monkeysaudio(self):
         self.failUnless(isinstance(
@@ -182,22 +191,30 @@ class TFile(TestCase):
 
 add(TFile)
 
-class TMutagen(TestCase):
-    uses_mmap = False
+class TFileUpperExt(TestCase):
+    FILES = [(os.path.join("tests", "data", "empty.ofr"), OptimFROG),
+             (os.path.join("tests", "data", "sv5_header.mpc"), Musepack),
+             (os.path.join("tests", "data", "silence-3.wma"), ASF),
+             (os.path.join("tests", "data", "truncated-64bit.mp4"), MP4),
+             (os.path.join("tests", "data", "silence-44-s.flac"), FLAC),
+             ]
+             
+    def setUp(self):
+        checks = []
+        for (original, instance) in self.FILES:
+            ext = original.rsplit(".", 1)[-1]
+            fd, filename = mkstemp(suffix='.'+ext.upper())
+            os.close(fd)
+            shutil.copy(original, filename)
+            checks.append((filename, instance))
+        self.checks = checks
 
-    def test_not_prerelease(self):
-        from mutagen import version
-        from os.path import dirname, basename, abspath
-        trunk = dirname(dirname(abspath(__file__)))
-        definitely_trunk = os.path.join(trunk, "..", "trunk")
-        if os.path.isdir(definitely_trunk):
-            if basename(trunk) != "trunk":
-                self.failIf(
-                    version[-1] < 0, "mutagen.version is a prerelease.")
-            else:
-                self.failIf(
-                    version[-1] >= 0, "mutagen.version is not a prerelease.")
-        else:
-            print "WARNING: Unable to check version correctness."
-            print "WARNING: %r does not exist." % definitely_trunk
-add(TMutagen)
+    def test_case_insensitive_ext(self):
+        for (path, instance) in self.checks:
+            self.failUnless(isinstance(File(path, options=[instance]), instance))
+
+    def tearDown(self):
+        for (path, instance) in self.checks:
+            os.unlink(path)
+
+add(TFileUpperExt)
