@@ -45,7 +45,7 @@ class TAPEWriter(TestCase):
         tag.save(SAMPLE + ".new")
         tag.save(SAMPLE + ".justtag")
         tag.save(SAMPLE + ".tag_at_start")
-        fileobj = file(SAMPLE + ".tag_at_start", "ab")
+        fileobj = open(SAMPLE + ".tag_at_start", "ab")
         fileobj.write("tag garbage" * 1000)
         fileobj.close()
         self.tag = mutagen.apev2.APEv2(SAMPLE + ".new")
@@ -120,6 +120,13 @@ class TAPEWriter(TestCase):
         self.failUnless("FoObaR" in tag.keys())
         self.failIf("foobar" in tag.keys())
 
+    def test_unicode_key(self):
+        # http://code.google.com/p/mutagen/issues/detail?id=123
+        tag = mutagen.apev2.APEv2(SAMPLE + ".new")
+        tag["abc"] = u'\xf6\xe4\xfc'
+        tag[u"cba"] = "abc"
+        tag.save()
+
     def tearDown(self):
         os.unlink(SAMPLE + ".new")
         os.unlink(BROKEN + ".new")
@@ -133,13 +140,13 @@ class TAPEv2ThenID3v1Writer(TAPEWriter):
 
     def setUp(self):
         super(TAPEv2ThenID3v1Writer, self).setUp()
-        f = file(SAMPLE + ".new", "ab+")
+        f = open(SAMPLE + ".new", "ab+")
         f.write("TAG" + "\x00" * 125)
         f.close()
-        f = file(BROKEN + ".new", "ab+")
+        f = open(BROKEN + ".new", "ab+")
         f.write("TAG" + "\x00" * 125)
         f.close()
-        f = file(SAMPLE + ".justtag", "ab+")
+        f = open(SAMPLE + ".justtag", "ab+")
         f.write("TAG" + "\x00" * 125)
         f.close()
 
@@ -173,12 +180,6 @@ class TAPEv2(TestCase):
         self.failUnlessEqual(self.audio["test"], "foobar\x00quuxbarz")
         self.failUnless(isinstance(self.audio["test"], APETextValue))
 
-    def test_guess_text_list(self):
-        from mutagen.apev2 import APETextValue
-        self.audio["test"] = [u"foobar", "quuxbarz"]
-        self.failUnlessEqual(self.audio["test"], "foobar\x00quuxbarz")
-        self.failUnless(isinstance(self.audio["test"], APETextValue))
-
     def test_guess_utf8(self):
         from mutagen.apev2 import APETextValue
         self.audio["test"] = "foobar"
@@ -218,6 +219,11 @@ class TAPEv2(TestCase):
         self.failUnlessEqual(
             self.audio.items(), zip(self.audio.keys(), self.audio.values()))
 
+    def test_invalid_keys(self):
+        self.failUnlessRaises(KeyError, self.audio.__getitem__, "\x00")
+        self.failUnlessRaises(KeyError, self.audio.__setitem__, "\x00", "")
+        self.failUnlessRaises(KeyError, self.audio.__delitem__, "\x00")
+
     def test_dictlike(self):
         self.failUnless(self.audio.get("track"))
         self.failUnless(self.audio.get("Track"))
@@ -253,7 +259,7 @@ class TAPEv2ThenID3v1(TAPEv2):
 
     def setUp(self):
         super(TAPEv2ThenID3v1, self).setUp()
-        f = file(self.filename, "ab+")
+        f = open(self.filename, "ab+")
         f.write("TAG" + "\x00" * 125)
         f.close()
         self.audio = APEv2(self.filename)
