@@ -2,7 +2,7 @@
 import shutil, os
 from tests import TestCase, add
 from mutagen.id3 import ID3, TIT2, ID3NoHeaderError
-from mutagen.flac import to_int_be, Padding, VCFLACDict, MetadataBlock
+from mutagen.flac import to_int_be, Padding, VCFLACDict, MetadataBlock, error
 from mutagen.flac import StreamInfo, SeekTable, CueSheet, FLAC, delete, Picture
 from tests.test__vorbis import TVCommentDict, VComment
 try: from os.path import devnull
@@ -64,10 +64,15 @@ class TStreamInfo(TestCase):
 
     data = ('\x12\x00\x12\x00\x00\x00\x0e\x005\xea\n\xc4H\xf0\x00\xca0'
             '\x14(\x90\xf9\xe1)2\x13\x01\xd4\xa7\xa9\x11!8\xab\x91')
+    data_invalid = len(data) * '\x00'
 
     def setUp(self):
         self.i = StreamInfo(self.data)
 
+    def test_invalid(self):
+        # http://code.google.com/p/mutagen/issues/detail?id=117
+        self.failUnlessRaises(error, StreamInfo, self.data_invalid)
+
     def test_blocksize(self):
         self.failUnlessEqual(self.i.max_blocksize, 4608)
         self.failUnlessEqual(self.i.min_blocksize, 4608)
@@ -196,7 +201,7 @@ class TFLAC(TestCase):
     NEW = SAMPLE + ".new"
     def setUp(self):
         shutil.copy(self.SAMPLE, self.NEW)
-        self.failUnlessEqual(file(self.SAMPLE).read(), file(self.NEW).read())
+        self.failUnlessEqual(open(self.SAMPLE).read(), open(self.NEW).read())
         self.flac = FLAC(self.NEW)
 
     def test_delete(self):
@@ -229,7 +234,7 @@ class TFLAC(TestCase):
     def test_write_nochange(self):
         f = FLAC(self.NEW)
         f.save()
-        self.failUnlessEqual(file(self.SAMPLE).read(), file(self.NEW).read())
+        self.failUnlessEqual(open(self.SAMPLE).read(), open(self.NEW).read())
 
     def test_write_changetitle(self):
         f = FLAC(self.NEW)
@@ -288,6 +293,12 @@ class TFLAC(TestCase):
         self.failUnless(f.tags == [("foo", "bar")])
         self.failUnlessRaises(ValueError, f.add_tags)
 
+    def test_ooming_vc_header(self):
+        # issue 112: Malformed FLAC Vorbis header causes out of memory error
+        # http://code.google.com/p/mutagen/issues/detail?id=112
+        self.assertRaises(IOError, FLAC, os.path.join('tests', 'data',
+                                                      'ooming-header.flac'))
+
     def test_with_real_flac(self):
         if not have_flac: return
         self.flac["faketag"] = "foobar" * 1000
@@ -390,6 +401,9 @@ class TFLAC(TestCase):
     def test_variable_block_size(self):
         FLAC(os.path.join("tests", "data", "variable-block.flac"))
 
+    def test_load_flac_with_application_block(self):
+        FLAC(os.path.join("tests", "data", "flac_application.flac"))
+
     def tearDown(self):
         os.unlink(self.NEW)
 
@@ -406,6 +420,53 @@ class TFLACFile(TestCase):
 
 add(TFLACFile)
 
+class TFLACBadBlockSize(TestCase):
+    uses_mmap = False
+    TOO_SHORT = os.path.join("tests", "data", "52-too-short-block-size.flac")
+    TOO_SHORT_2 = os.path.join("tests", "data",
+                               "106-short-picture-block-size.flac")
+    OVERWRITTEN = os.path.join("tests", "data", "52-overwritten-metadata.flac")
+    INVAL_INFO = os.path.join("tests", "data", "106-invalid-streaminfo.flac")
+
+    def test_too_short_read(self):
+        flac = FLAC(self.TOO_SHORT)
+        self.failUnlessEqual(flac["artist"], ["Tunng"])
+
+    def test_too_short_read_picture(self):
+        flac = FLAC(self.TOO_SHORT_2)
+        self.failUnlessEqual(flac.pictures[0].width, 10)
+
+    def test_overwritten_read(self):
+        flac = FLAC(self.OVERWRITTEN)
+        self.failUnlessEqual(flac["artist"], ["Giora Feidman"])
+
+    def test_inval_streaminfo(self):
+        self.assertRaises(error, FLAC, self.INVAL_INFO)
+
+add(TFLACBadBlockSize)
+
+class TFLACBadBlockSizeWrite(TestCase):
+    uses_mmap = False
+    TOO_SHORT = os.path.join("tests", "data", "52-too-short-block-size.flac")
+    NEW = TOO_SHORT + ".new"
+
+    def setUp(self):
+        shutil.copy(self.TOO_SHORT, self.NEW)
+
+    def test_write_reread(self):
+        flac = FLAC(self.NEW)
+        del(flac["artist"])
+        flac.save()
+        flac2 = FLAC(self.NEW)
+        self.failUnlessEqual(flac["title"], flac2["title"])
+        data = open(self.NEW, "rb").read(1024)
+        self.failIf("Tunng" in data)
+
+    def tearDown(self):
+        os.unlink(self.NEW)
+        
+add(TFLACBadBlockSizeWrite)
+
 class CVE20074619(TestCase):
     uses_mmap = True
 
