@@ -7,9 +7,6 @@ from StringIO import StringIO
 import warnings
 warnings.simplefilter('error', ID3Warning)
 
-try: from sets import Set as set
-except ImportError: pass
-
 _22 = ID3(); _22.version = (2,2,0)
 _23 = ID3(); _23.version = (2,3,0)
 _24 = ID3(); _24.version = (2,4,0)
@@ -87,12 +84,12 @@ class ID3Loading(TestCase):
 
     def test_header_empty(self):
         id3 = ID3()
-        id3._ID3__fileobj = file(self.empty, 'rb')
+        id3._ID3__fileobj = open(self.empty, 'rb')
         self.assertRaises(EOFError, id3._ID3__load_header)
 
     def test_header_silence(self):
         id3 = ID3()
-        id3._ID3__fileobj = file(self.silence, 'rb')
+        id3._ID3__fileobj = open(self.silence, 'rb')
         id3._ID3__load_header()
         self.assertEquals(id3.version, (2,3,0))
         self.assertEquals(id3.size, 1314)
@@ -276,6 +273,9 @@ class ID3Tags(TestCase):
         self.assertRaises(
             ID3JunkFrameError, Frames["SYLT"].fromData, _24, 0x0,
             "\x00eng\x01description\x00foobar")
+        self.assertRaises(
+            ID3JunkFrameError, Frames["SYLT"].fromData, _24, 0x0,
+            "\x00eng\x01description\x00foobar\x00\xFF\xFF\xFF")
 
     def test_extradata(self):
         from mutagen.id3 import RVRB, RBUF
@@ -390,12 +390,12 @@ class TestWriteID3v1(TestCase):
         self.audio = ID3(self.filename)
 
     def failIfV1(self):
-        fileobj = file(self.filename, "rb")
+        fileobj = open(self.filename, "rb")
         fileobj.seek(-128, 2)
         self.failIf(fileobj.read(3) == "TAG")
 
     def failUnlessV1(self):
-        fileobj = file(self.filename, "rb")
+        fileobj = open(self.filename, "rb")
         fileobj.seek(-128, 2)
         self.failUnless(fileobj.read(3) == "TAG")
 
@@ -1172,6 +1172,64 @@ class UpdateTo24(TestCase):
 
 add(UpdateTo24)
 
+
+class Issue97_UpgradeUnknown23(TestCase):
+    SILENCE = os.path.join("tests", "data", "97-unknown-23-update.mp3")
+    def setUp(self):
+        from tempfile import mkstemp
+        fd, self.filename = mkstemp(suffix='.mp3')
+        os.close(fd)
+        shutil.copy(self.SILENCE, self.filename)
+
+    def test_unknown(self):
+        from mutagen.id3 import TPE1
+        orig = ID3(self.filename)
+        self.failUnlessEqual(orig.version, (2, 3, 0))
+
+        # load a 2.3 file and pretend we don't support TIT2
+        unknown = ID3(self.filename, known_frames={"TPE1": TPE1},
+                      translate=False)
+
+        # TIT2 ends up in unknown_frames
+        self.failUnlessEqual(unknown.unknown_frames[0][:4], "TIT2")
+
+         # frame should be different now
+        orig_unknown = unknown.unknown_frames[0]
+        unknown.update_to_v24()
+        self.failIfEqual(unknown.unknown_frames[0], orig_unknown)
+
+        # save as 2.4
+        unknown.save()
+
+        # load again with support for TIT2, all should be there again
+        new = ID3(self.filename)
+        self.failUnlessEqual(new.version, (2, 4, 0))
+        self.failUnlessEqual(new["TIT2"].text, orig["TIT2"].text)
+        self.failUnlessEqual(new["TPE1"].text, orig["TPE1"].text)
+
+    def test_double_update(self):
+        from mutagen.id3 import TPE1
+        orig = ID3(self.filename)
+        unknown = ID3(self.filename, known_frames={"TPE1": TPE1})
+        # Make sure the data doesn't get updated again
+        unknown.update_to_v24()
+        unknown.unknown_frames = ["foobar"]
+        unknown.update_to_v24()
+        self.failUnless(unknown.unknown_frames)
+
+    def test_unkown_invalid(self):
+        f = ID3(self.filename, translate=False)
+        f.unknown_frames = ["foobar", "\xff"*50]
+        # throw away invalid frames
+        f.update_to_v24()
+        self.failIf(f.unknown_frames)
+
+    def tearDown(self):
+        os.unlink(self.filename)
+
+add(Issue97_UpgradeUnknown23)
+
+
 class Genres(TestCase):
     uses_mmap = False
 
@@ -1442,7 +1500,7 @@ class OddWrites(TestCase):
 
     def test_toemptyfile(self):
         os.unlink(self.newsilence)
-        file(self.newsilence, "wb").close()
+        open(self.newsilence, "wb").close()
         ID3(self.silence).save(self.newsilence)
 
     def test_tononfile(self):
@@ -1451,12 +1509,12 @@ class OddWrites(TestCase):
 
     def test_1bfile(self):
         os.unlink(self.newsilence)
-        f = file(self.newsilence, "wb")
+        f = open(self.newsilence, "wb")
         f.write("!")
         f.close()
         ID3(self.silence).save(self.newsilence)
         self.assert_(os.path.getsize(self.newsilence) > 1)
-        self.assertEquals(file(self.newsilence, "rb").read()[-1], "!")
+        self.assertEquals(open(self.newsilence, "rb").read()[-1], "!")
 
     def tearDown(self):
         try: os.unlink(self.newsilence)
@@ -1548,7 +1606,9 @@ class WriteRoundtrip(TestCase):
         self.assertEquals(open(self.newsilence).read(10), '')
 
     def test_delete_invalid_zero(self):
-        open(self.newsilence, 'wb').write('ID3\x04\x00\x00\x00\x00\x00\x00abc')
+        f = open(self.newsilence, 'wb')
+        f.write('ID3\x04\x00\x00\x00\x00\x00\x00abc')
+        f.close()
         ID3(self.newsilence).delete()
         self.assertEquals(open(self.newsilence).read(10), 'abc')
 
@@ -1577,7 +1637,7 @@ class WriteForEyeD3(TestCase):
     def setUp(self):
         shutil.copy(self.silence, self.newsilence)
         # remove ID3v1 tag
-        f = file(self.newsilence, "rb+")
+        f = open(self.newsilence, "rb+")
         f.seek(-128, 2)
         f.truncate()
         f.close()
@@ -1697,6 +1757,48 @@ class TimeStampTextFrame(TestCase):
         frame = self.Frame(encoding=0, text=[u'1987', u'1988'])
         self.failUnlessEqual(frame, unicode(frame))
 
+class Issue69_BadV1Year(TestCase):
+    uses_mmap = False
+
+    def test_missing_year(self):
+        from mutagen.id3 import ParseID3v1
+        tag = ParseID3v1('ABCTAGhello world\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xff')
+        self.failUnlessEqual(tag["TIT2"], "hello world")
+
+    def test_short_year(self):
+        from mutagen.id3 import ParseID3v1
+        tag = ParseID3v1('XTAGhello world\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x001\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xff')
+        self.failUnlessEqual(tag["TIT2"], "hello world")
+        self.failUnlessEqual(tag["TDRC"], "0001")
+
+    def test_none(self):
+        from mutagen.id3 import ParseID3v1, MakeID3v1, TDRC
+        s = MakeID3v1(dict())
+        self.failUnlessEqual(len(s), 128)
+        tag = ParseID3v1(s)
+        self.failIf("TDRC" in tag)
+
+    def test_empty(self):
+        from mutagen.id3 import ParseID3v1, MakeID3v1, TDRC
+        s = MakeID3v1(dict(TDRC=""))
+        self.failUnlessEqual(len(s), 128)
+        tag = ParseID3v1(s)
+        self.failIf("TDRC" in tag)
+
+    def test_short(self):
+        from mutagen.id3 import ParseID3v1, MakeID3v1, TDRC
+        s = MakeID3v1(dict(TDRC="1"))
+        self.failUnlessEqual(len(s), 128)
+        tag = ParseID3v1(s)
+        self.failUnlessEqual(tag["TDRC"], "0001")
+
+    def test_long(self):
+        from mutagen.id3 import ParseID3v1, MakeID3v1, TDRC
+        s = MakeID3v1(dict(TDRC="123456789"))
+        self.failUnlessEqual(len(s), 128)
+        tag = ParseID3v1(s)
+        self.failUnlessEqual(tag["TDRC"], "1234")
+
 add(ID3Loading)
 add(ID3GetSetDel)
 add(BitPaddedIntTest)
@@ -1715,6 +1817,7 @@ add(FrameIDValidate)
 add(BadTYER)
 add(BadPOPM)
 add(TimeStampTextFrame)
+add(Issue69_BadV1Year)
 
 try: import eyeD3
 except ImportError: pass
