@@ -4,13 +4,15 @@ import shutil
 from unittest import TestCase
 from cStringIO import StringIO
 from tests import add
-from mutagen.mp3 import MP3, error as MP3Error, delete, MPEGInfo
+from mutagen.mp3 import MP3, error as MP3Error, delete, MPEGInfo, EasyMP3
 from mutagen.id3 import ID3
 from tempfile import mkstemp
 
 class TMP3(TestCase):
     silence = os.path.join('tests', 'data', 'silence-44-s.mp3')
     silence_nov2 = os.path.join('tests', 'data', 'silence-44-s-v1.mp3')
+    silence_mpeg2 = os.path.join('tests', 'data', 'silence-44-s-mpeg2.mp3')
+    silence_mpeg25 = os.path.join('tests', 'data', 'silence-44-s-mpeg25.mp3')
 
     def setUp(self):
         original = os.path.join("tests", "data", "silence-44-s.mp3")
@@ -19,28 +21,40 @@ class TMP3(TestCase):
         shutil.copy(original, self.filename)
         self.mp3 = MP3(self.filename)
         self.mp3_2 = MP3(self.silence_nov2)
+        self.mp3_3 = MP3(self.silence_mpeg2)
+        self.mp3_4 = MP3(self.silence_mpeg25)
 
     def test_mode(self):
         from mutagen.mp3 import JOINTSTEREO
         self.failUnlessEqual(self.mp3.info.mode, JOINTSTEREO)
         self.failUnlessEqual(self.mp3_2.info.mode, JOINTSTEREO)
+        self.failUnlessEqual(self.mp3_3.info.mode, JOINTSTEREO)
+        self.failUnlessEqual(self.mp3_4.info.mode, JOINTSTEREO)
 
     def test_id3(self):
         self.failUnlessEqual(self.mp3.tags, ID3(self.silence))
         self.failUnlessEqual(self.mp3_2.tags, ID3(self.silence_nov2))
 
     def test_length(self):
-        self.failUnlessEqual(int(round(self.mp3.info.length)), 4)
-        self.failUnlessEqual(int(round(self.mp3_2.info.length)), 4)
+        self.assertAlmostEquals(self.mp3.info.length, 3.77, 2)
+        self.assertAlmostEquals(self.mp3_2.info.length, 3.77, 2)
+        self.assertAlmostEquals(self.mp3_3.info.length, 3.77, 2)
+        self.assertAlmostEquals(self.mp3_4.info.length, 3.84, 2)
     def test_version(self):
         self.failUnlessEqual(self.mp3.info.version, 1)
         self.failUnlessEqual(self.mp3_2.info.version, 1)
+        self.failUnlessEqual(self.mp3_3.info.version, 2)
+        self.failUnlessEqual(self.mp3_4.info.version, 2.5)
     def test_layer(self):
         self.failUnlessEqual(self.mp3.info.layer, 3)
         self.failUnlessEqual(self.mp3_2.info.layer, 3)
+        self.failUnlessEqual(self.mp3_3.info.layer, 3)
+        self.failUnlessEqual(self.mp3_4.info.layer, 3)
     def test_bitrate(self):
         self.failUnlessEqual(self.mp3.info.bitrate, 32000)
         self.failUnlessEqual(self.mp3_2.info.bitrate, 32000)
+        self.failUnlessEqual(self.mp3_3.info.bitrate, 18191)
+        self.failUnlessEqual(self.mp3_4.info.bitrate, 9300)
 
     def test_notmp3(self):
         self.failUnlessRaises(MP3Error, MP3, "README")
@@ -48,6 +62,8 @@ class TMP3(TestCase):
     def test_sketchy(self):
         self.failIf(self.mp3.info.sketchy)
         self.failIf(self.mp3_2.info.sketchy)
+        self.failIf(self.mp3_3.info.sketchy)
+        self.failIf(self.mp3_4.info.sketchy)
 
     def test_sketchy_notmp3(self):
         notmp3 = MP3(os.path.join("tests", "data", "silence-44-s.flac"))
@@ -62,12 +78,12 @@ class TMP3(TestCase):
 
     def test_xing(self):
         mp3 = MP3(os.path.join("tests", "data", "xing.mp3"))
-        self.failUnlessEqual(mp3.info.length, 26122)
+        self.failUnlessEqual(int(round(mp3.info.length)), 26122)
         self.failUnlessEqual(mp3.info.bitrate, 306)
 
     def test_vbri(self):
         mp3 = MP3(os.path.join("tests", "data", "vbri.mp3"))
-        self.failUnlessEqual(mp3.info.length, 222)
+        self.failUnlessEqual(int(round(mp3.info.length)), 222)
 
     def test_empty_xing(self):
         mp3 = MP3(os.path.join("tests", "data", "bad-xing.mp3"))
@@ -120,10 +136,48 @@ class TMPEGInfo(TestCase):
 
     def test_not_real_file(self):
         filename = os.path.join("tests", "data", "silence-44-s-v1.mp3")
-        fileobj = StringIO(file(filename, "rb").read(20))
+        fileobj = StringIO(open(filename, "rb").read(20))
         MPEGInfo(fileobj)
 
     def test_empty(self):
         fileobj = StringIO("")
         self.failUnlessRaises(IOError, MPEGInfo, fileobj)
 add(TMPEGInfo)
+
+class TEasyMP3(TestCase):
+    uses_mmap = False
+
+    def setUp(self):
+        original = os.path.join("tests", "data", "silence-44-s.mp3")
+        fd, self.filename = mkstemp(suffix='.mp3')
+        os.close(fd)
+        shutil.copy(original, self.filename)
+        self.mp3 = EasyMP3(self.filename)
+
+    def test_artist(self):
+        self.failUnless("artist" in self.mp3)
+
+    def test_no_composer(self):
+        self.failIf("composer" in self.mp3)
+
+    def test_length(self):
+        # http://code.google.com/p/mutagen/issues/detail?id=125
+        # easyid3, normal id3 and mpeg loading without tags should skip
+        # the tags and get the right offset of the first frame
+        easy = self.mp3.info
+        noneasy = MP3(self.filename).info
+        nonid3 = MPEGInfo(open(self.filename, "rb"))
+
+        self.failUnlessEqual(easy.length, noneasy.length)
+        self.failUnlessEqual(noneasy.length, nonid3.length)
+
+    def tearDown(self):
+        os.unlink(self.filename)
+add(TEasyMP3)
+
+class Issue72_TooShortFile(TestCase):
+    def test_load(self):
+        mp3 = MP3(os.path.join('tests', 'data', 'too-short.mp3'))
+        self.failUnlessEqual(mp3["TIT2"], "Track 10")
+        self.failUnlessAlmostEqual(mp3.info.length, 0.03, 2)
+add(Issue72_TooShortFile)
