@@ -6,7 +6,7 @@ from cStringIO import StringIO
 from tempfile import mkstemp
 from tests import TestCase, add
 from mutagen.mp4 import MP4, Atom, Atoms, MP4Tags, MP4Info, \
-     delete, MP4Cover, MP4MetadataError
+     delete, MP4Cover, MP4MetadataError, MP4FreeForm
 from mutagen._util import cdata
 try: from os.path import devnull
 except ImportError: devnull = "/dev/null"
@@ -24,6 +24,10 @@ class TAtom(TestCase):
                            "\x00\x00\x00\x00\x00\x00\x00\x08" + "\x00" * 8)
         self.failUnlessEqual(Atom(fileobj).length, 8)
 
+    def test_length_less_than_8(self):
+        fileobj = StringIO("\x00\x00\x00\x02atom")
+        self.assertRaises(MP4MetadataError, Atom, fileobj)
+
     def test_render_too_big(self):
         class TooBig(str):
             def __len__(self):
@@ -37,10 +41,24 @@ class TAtom(TestCase):
             data = Atom.render("data", data)
             self.failUnlessEqual(len(data), 4 + 4 + 8 + 4)
 
+    def test_non_top_level_length_0_is_invalid(self):
+        data = StringIO(struct.pack(">I4s", 0, "whee"))
+        self.assertRaises(MP4MetadataError, Atom, data, level=1)
+
     def test_length_0(self):
-        fileobj = StringIO("\x00\x00\x00\x00atom")
-        Atom(fileobj)
-        self.failUnlessEqual(fileobj.tell(), 8)
+        fileobj = StringIO("\x00\x00\x00\x00atom" + 40 * "\x00")
+        atom = Atom(fileobj)
+        self.failUnlessEqual(fileobj.tell(), 48)
+        self.failUnlessEqual(atom.length, 48)
+
+    def test_length_0_container(self):
+        data = StringIO(struct.pack(">I4s", 0, "moov") +
+                        Atom.render("data", "whee"))
+        atom = Atom(data)
+        self.failUnlessEqual(len(atom.children), 1)
+        self.failUnlessEqual(atom.length, 20)
+        self.failUnlessEqual(atom.children[-1].length, 12)
+
 add(TAtom)
 
 class TAtoms(TestCase):
@@ -48,7 +66,7 @@ class TAtoms(TestCase):
     filename = os.path.join("tests", "data", "has-tags.m4a")
 
     def setUp(self):
-        self.atoms = Atoms(file(self.filename, "rb"))
+        self.atoms = Atoms(open(self.filename, "rb"))
 
     def test___contains__(self):
         self.failUnless(self.atoms["moov"])
@@ -217,6 +235,30 @@ class TMP4Tags(TestCase):
         tags["trck"] = [(1, 2), (3, 4)]
         tags.pprint()
 
+    def test_freeform_data(self):
+        # http://code.google.com/p/mutagen/issues/detail?id=103
+        key = "----:com.apple.iTunes:Encoding Params"
+        value = ("vers\x00\x00\x00\x01acbf\x00\x00\x00\x01brat\x00\x01\xf4"
+                 "\x00cdcv\x00\x01\x05\x04")
+
+        data = ("\x00\x00\x00\x1cmean\x00\x00\x00\x00com.apple.iTunes\x00\x00"
+                "\x00\x1bname\x00\x00\x00\x00Encoding Params\x00\x00\x000data"
+                "\x00\x00\x00\x00\x00\x00\x00\x00vers\x00\x00\x00\x01acbf\x00"
+                "\x00\x00\x01brat\x00\x01\xf4\x00cdcv\x00\x01\x05\x04")
+
+        tags = self.wrap_ilst(Atom.render("----", data))
+        v = tags[key][0]
+        self.failUnlessEqual(v, value)
+        self.failUnlessEqual(v.dataformat, MP4FreeForm.FORMAT_DATA)
+
+        data = MP4Tags()._MP4Tags__render_freeform(key, v)
+        v = self.wrap_ilst(data)[key][0]
+        self.failUnlessEqual(v.dataformat, MP4FreeForm.FORMAT_DATA)
+
+        data = MP4Tags()._MP4Tags__render_freeform(key, value)
+        v = self.wrap_ilst(data)[key][0]
+        self.failUnlessEqual(v.dataformat, MP4FreeForm.FORMAT_TEXT)
+
 add(TMP4Tags)
 
 class TMP4(TestCase):
@@ -233,7 +275,7 @@ class TMP4(TestCase):
         self.failIf(value and value != NOTFOUND)
 
     def test_score(self):
-        fileobj = file(self.filename)
+        fileobj = open(self.filename)
         header = fileobj.read(128)
         self.failUnless(MP4.score(self.filename, fileobj, header))
 
@@ -266,7 +308,7 @@ class TMP4(TestCase):
         self.audio["\xa9nam"] = u"wheeee" * 10
         self.audio.save()
         # Reorder "free" and "ilst" atoms
-        fileobj = file(self.audio.filename, "rb+")
+        fileobj = open(self.audio.filename, "rb+")
         atoms = Atoms(fileobj)
         meta = atoms["moov", "udta", "meta"]
         meta_length1 = meta.length
@@ -280,7 +322,7 @@ class TMP4(TestCase):
         fileobj.seek(ilst.offset)
         fileobj.write(free_data + ilst_data)
         fileobj.close()
-        fileobj = file(self.audio.filename, "rb+")
+        fileobj = open(self.audio.filename, "rb+")
         atoms = Atoms(fileobj)
         meta = atoms["moov", "udta", "meta"]
         ilst = meta["ilst",]
@@ -291,7 +333,7 @@ class TMP4(TestCase):
         self.audio["\xa9nam"] = u"wheeee" * 11
         self.audio.save()
         # Check the order of "free" and "ilst" atoms
-        fileobj = file(self.audio.filename, "rb+")
+        fileobj = open(self.audio.filename, "rb+")
         atoms = Atoms(fileobj)
         fileobj.close()
         meta = atoms["moov", "udta", "meta"]
@@ -328,6 +370,13 @@ class TMP4(TestCase):
     def test_freeforms(self):
         self.set_key('----:net.sacredchao.Mutagen:test key', ["whee", "uhh"])
 
+    def test_freeform_bin(self):
+        self.set_key('----:net.sacredchao.Mutagen:test key', [
+            MP4FreeForm('woooo', MP4FreeForm.FORMAT_TEXT),
+            MP4FreeForm('hoooo', MP4FreeForm.FORMAT_DATA),
+            MP4FreeForm('boooo'),
+        ])
+
     def test_tracknumber(self):
         self.set_key('trkn', [(1, 10)])
         self.set_key('trkn', [(1, 10), (5, 20)], faad=False)
@@ -427,7 +476,7 @@ class TMP4(TestCase):
         self.set_key("foob", [u"A test"])
 
     def __read_offsets(self, filename):
-        fileobj = file(filename, 'rb')
+        fileobj = open(filename, 'rb')
         atoms = Atoms(fileobj)
         moov = atoms['moov']
         samples = []
@@ -474,9 +523,8 @@ class TMP4(TestCase):
     def tearDown(self):
         os.unlink(self.filename)
 
-class TMP4HasTags(TMP4):
-    original = os.path.join("tests", "data", "has-tags.m4a")
 
+class TMP4HasTags(TMP4):
     def test_save_simple(self):
         self.audio.save()
         self.faad()
@@ -490,6 +538,20 @@ class TMP4HasTags(TMP4):
     def test_has_tags(self):
         self.failUnless(self.audio.tags)
 
+    def test_not_my_file(self):
+        self.failUnlessRaises(
+            IOError, MP4, os.path.join("tests", "data", "empty.ogg"))
+
+
+class TMP4Datatypes(TMP4HasTags):
+    original = os.path.join("tests", "data", "has-tags.m4a")
+
+    def test_has_freeform(self):
+        key = "----:com.apple.iTunes:iTunNORM"
+        self.failUnless(key in self.audio.tags)
+        ff = self.audio.tags[key]
+        self.failUnlessEqual(ff[0].dataformat, MP4FreeForm.FORMAT_TEXT)
+
     def test_has_covr(self):
         self.failUnless('covr' in self.audio.tags)
         covr = self.audio.tags['covr']
@@ -497,11 +559,21 @@ class TMP4HasTags(TMP4):
         self.failUnlessEqual(covr[0].imageformat, MP4Cover.FORMAT_PNG)
         self.failUnlessEqual(covr[1].imageformat, MP4Cover.FORMAT_JPEG)
 
-    def test_not_my_file(self):
-        self.failUnlessRaises(
-            IOError, MP4, os.path.join("tests", "data", "empty.ogg"))
+add(TMP4Datatypes)
 
-add(TMP4HasTags)
+
+class TMP4CovrWithName(TMP4):
+    # http://bugs.musicbrainz.org/ticket/5894
+    original = os.path.join("tests", "data", "covr-with-name.m4a")
+
+    def test_has_covr(self):
+        self.failUnless('covr' in self.audio.tags)
+        covr = self.audio.tags['covr']
+        self.failUnlessEqual(len(covr), 2)
+        self.failUnlessEqual(covr[0].imageformat, MP4Cover.FORMAT_PNG)
+        self.failUnlessEqual(covr[1].imageformat, MP4Cover.FORMAT_JPEG)
+
+add(TMP4CovrWithName)
 
 class TMP4HasTags64Bit(TMP4HasTags):
     original = os.path.join("tests", "data", "truncated-64bit.mp4")
