@@ -5,6 +5,7 @@ import shutil
 from StringIO import StringIO
 from tests import TestCase, add
 from mutagen.ogg import OggPage, error as OggError
+from mutagen._util import cdata
 from tempfile import mkstemp
 try: from os.path import devnull
 except ImportError: devnull = "/dev/null"
@@ -13,7 +14,7 @@ class TOggPage(TestCase):
     uses_mmap = False
 
     def setUp(self):
-        self.fileobj = file(os.path.join("tests", "data", "empty.ogg"), "rb")
+        self.fileobj = open(os.path.join("tests", "data", "empty.ogg"), "rb")
         self.page = OggPage(self.fileobj)
 
         pages = [OggPage(), OggPage(), OggPage()]
@@ -126,10 +127,10 @@ class TOggPage(TestCase):
             os.close(fd)
             shutil.copy(os.path.join("tests", "data", "multipagecomment.ogg"),
                         filename)
-            fileobj = file(filename, "rb+")
+            fileobj = open(filename, "rb+")
             OggPage.renumber(fileobj, 1002429366L, 20)
             fileobj.close()
-            fileobj = file(filename, "rb+")
+            fileobj = open(filename, "rb+")
             OggPage.renumber(fileobj, 1002429366L, 0)
             fileobj.close()
         finally:
@@ -205,7 +206,7 @@ class TOggPage(TestCase):
         self.failUnlessEqual(OggPage.to_packets(pages), packets)
 
     def test_random_data_roundtrip(self):
-        try: random_file = file("/dev/urandom", "rb")
+        try: random_file = open("/dev/urandom", "rb")
         except (IOError, OSError):
             print "WARNING: Random data round trip test disabled."
             return
@@ -323,9 +324,42 @@ class TOggPage(TestCase):
         data = StringIO("if you think this is an Ogg, you're crazy")
         self.failUnlessRaises(OggError, OggPage.find_last, data, 0)
 
+    # Disabled because GStreamer will write Oggs with bad data,
+    # which we need to make a best guess for.
+    #
+    #def test_find_last_invalid_sync(self):
+    #    data = StringIO("if you think this is an OggS, you're crazy")
+    #    self.failUnlessRaises(OggError, OggPage.find_last, data, 0)
+
     def test_find_last_invalid_sync(self):
         data = StringIO("if you think this is an OggS, you're crazy")
-        self.failUnlessRaises(OggError, OggPage.find_last, data, 0)
+        page = OggPage.find_last(data, 0)
+        self.failIf(page)
+
+    def test_crc_py25(self):
+        # Make sure page.write can handle both signed/unsigned int
+        # return values of crc32.
+        # http://code.google.com/p/mutagen/issues/detail?id=63
+        # http://docs.python.org/library/zlib.html#zlib.crc32
+
+        import zlib
+        old_crc = zlib.crc32
+        def zlib_uint(*args):
+            return (old_crc(*args) & 0xffffffff)
+        def zlib_int(*args):
+            return cdata.int_be(cdata.to_uint_be(old_crc(*args) & 0xffffffff))
+
+        try:
+            page = OggPage()
+            page.packets = ["abc"]
+            zlib.crc32 = zlib_uint
+            uint_data = page.write()
+            zlib.crc32 = zlib_int
+            int_data  = page.write()
+        finally:
+            zlib.crc32 = old_crc
+
+        self.failUnlessEqual(uint_data, int_data)
 
     def tearDown(self):
         self.fileobj.close()
@@ -333,7 +367,7 @@ add(TOggPage)
 
 class TOggFileType(TestCase):
     def scan_file(self):
-        fileobj = file(self.filename, "rb")
+        fileobj = open(self.filename, "rb")
         try:
             try:
                 while True:
@@ -443,6 +477,8 @@ class TOggFileType(TestCase):
             self.failIf(value and value != NOTFOUND,
                         "ogginfo failed on %s" % filename)
         if have_oggz_validate:
+            if filename.endswith(".opus") and not have_oggz_validate_opus:
+                return
             value = os.system(
                 "oggz-validate %s > %s" % (filename, devnull))
             self.failIf(value and value != NOTFOUND,
@@ -472,6 +508,9 @@ class TOggFileType(TestCase):
         self.audio["foobar"] = "foobar" * 1000
         self.audio.save()
         self.ogg_reference(self.filename)
+
+    def test_mime_secondary(self):
+        self.failUnless('application/ogg' in self.audio.mime)
  
     def tearDown(self):
         os.unlink(self.filename)
@@ -483,6 +522,18 @@ if os.system("ogginfo 2> %s > %s" % (devnull, devnull)) == NOTFOUND:
     have_ogginfo = False
     print "WARNING: Skipping ogginfo reference tests."
 have_oggz_validate = True
+have_oggz_validate_opus = True
 if os.system("oggz-validate 2> %s > %s" % (devnull, devnull)) == NOTFOUND:
     have_oggz_validate = False
     print "WARNING: Skipping oggz-validate reference tests."
+else:
+    f = os.popen("oggz-validate --version")
+    try:
+        version_string = f.read()
+        version_part = version_string.split()[-1]
+        version = tuple(map(int, version_part.split(".")))
+        if version <= (0, 9, 9):
+            have_oggz_validate_opus = False
+            print "WARNING: Skipping oggz-validate reference tests for opus"
+    finally:
+        f.close()
