@@ -19,9 +19,13 @@ class TOggTheora(TOggFileType):
         os.close(fd)
         shutil.copy(original, self.filename)
         self.audio = OggTheora(self.filename)
+        self.audio2 = OggTheora(
+            os.path.join("tests", "data", "sample_length.oggtheora"))
+        self.audio3 = OggTheora(
+            os.path.join("tests", "data", "sample_bitrate.oggtheora"))
 
     def test_theora_bad_version(self):
-        page = OggPage(file(self.filename, "rb"))
+        page = OggPage(open(self.filename, "rb"))
         packet = page.packets[0]
         packet = packet[:7] + "\x03\x00" + packet[9:]
         page.packets = [packet]
@@ -29,7 +33,7 @@ class TOggTheora(TOggFileType):
         self.failUnlessRaises(IOError, OggTheoraInfo, fileobj)
 
     def test_theora_not_first_page(self):
-        page = OggPage(file(self.filename, "rb"))
+        page = OggPage(open(self.filename, "rb"))
         page.first = False
         fileobj = StringIO(page.write())
         self.failUnlessRaises(IOError, OggTheoraInfo, fileobj)
@@ -47,10 +51,17 @@ class TOggTheora(TOggFileType):
 
     def test_length(self):
         self.failUnlessAlmostEqual(5.5, self.audio.info.length, 1)
+        self.failUnlessAlmostEqual(0.75, self.audio2.info.length, 2)
+
+    def test_bitrate(self):
+        self.failUnlessEqual(16777215, self.audio3.info.bitrate)
 
     def test_module_delete(self):
         delete(self.filename)
         self.scan_file()
         self.failIf(OggTheora(self.filename).tags)
 
+    def test_mime(self):
+        self.failUnless("video/x-theora" in self.audio.mime)
+
 add(TOggTheora)
