@@ -34,12 +34,12 @@ class TOggVorbis(TOggFileType):
         self.failUnlessEqual(44100, self.audio.info.sample_rate)
 
     def test_invalid_not_first(self):
-        page = OggPage(file(self.filename, "rb"))
+        page = OggPage(open(self.filename, "rb"))
         page.first = False
         self.failUnlessRaises(IOError, OggVorbisInfo, StringIO(page.write()))
 
     def test_avg_bitrate(self):
-        page = OggPage(file(self.filename, "rb"))
+        page = OggPage(open(self.filename, "rb"))
         packet = page.packets[0]
         packet = (packet[:16] + "\x00\x00\x01\x00" + "\x00\x00\x00\x00" +
                   "\x00\x00\x00\x00" + packet[28:])
@@ -48,7 +48,7 @@ class TOggVorbis(TOggFileType):
         self.failUnlessEqual(info.bitrate, 32768)
 
     def test_overestimated_bitrate(self):
-        page = OggPage(file(self.filename, "rb"))
+        page = OggPage(open(self.filename, "rb"))
         packet = page.packets[0]
         packet = (packet[:16] + "\x00\x00\x01\x00" + "\x00\x00\x00\x01" +
                   "\x00\x00\x00\x00" + packet[28:])
@@ -57,7 +57,7 @@ class TOggVorbis(TOggFileType):
         self.failUnlessEqual(info.bitrate, 65536)
 
     def test_underestimated_bitrate(self):
-        page = OggPage(file(self.filename, "rb"))
+        page = OggPage(open(self.filename, "rb"))
         packet = page.packets[0]
         packet = (packet[:16] + "\x00\x00\x01\x00" + "\x01\x00\x00\x00" +
                   "\x00\x00\x01\x00" + packet[28:])
@@ -66,7 +66,7 @@ class TOggVorbis(TOggFileType):
         self.failUnlessEqual(info.bitrate, 65536)
 
     def test_negative_bitrate(self):
-        page = OggPage(file(self.filename, "rb"))
+        page = OggPage(open(self.filename, "rb"))
         packet = page.packets[0]
         packet = (packet[:16] + "\xff\xff\xff\xff" + "\xff\xff\xff\xff" +
                   "\xff\xff\xff\xff" + packet[28:])
