@@ -32,7 +32,7 @@ class TTrueAudio(TestCase):
         self.failIf(self.audio.tags)
 
     def test_pprint(self):
-        self.audio.pprint()
+        self.failUnless(self.audio.pprint())
 
     def test_save_reload(self):
         try:
@@ -48,4 +48,7 @@ class TTrueAudio(TestCase):
         finally:
             os.unlink(filename)
 
+    def test_mime(self):
+        self.failUnless("audio/x-tta" in self.audio.mime)
+
 add(TTrueAudio)
